@@ -377,7 +377,6 @@ checkRuleset(modConfData_t *modConf)
 	DEFiRet;
 
 	modConf->pBindRuleset = NULL;	/* assume default ruleset */
-dbgprintf("DDDD: impstats ruleset %s\n", modConf->pszBindRuleset);
 
 	if(modConf->pszBindRuleset == NULL)
 		FINALIZE;
@@ -390,7 +389,6 @@ dbgprintf("DDDD: impstats ruleset %s\n", modConf->pszBindRuleset);
 	CHKiRet(localRet);
 	modConf->pBindRuleset = pRuleset;
 finalize_it:
-dbgprintf("DDDD: impstats ruleset ptr %p\n", modConf->pBindRuleset);
 	RETiRet;
 }
 
@@ -459,12 +457,14 @@ BEGINrunInput
 CODESTARTrunInput
 	/* this is an endless loop - it is terminated when the thread is
 	 * signalled to do so. This, however, is handled by the framework,
-	 * right into the sleep below. Note that we DELIBERATLY output
-	 * final set of stats counters on termination request. Depending
-	 * on configuration, they may not make it to the final destination...
+	 * right into the sleep below.
 	 */
-	while(glbl.GetGlobalInputTermState() == 0) {
+	while(1) {
 		srSleep(runModConf->iStatsInterval, 0); /* seconds, micro seconds */
+
+		if(glbl.GetGlobalInputTermState() == 1)
+			break; /* terminate input! */
+
 		DBGPRINTF("impstats: woke up, generating messages\n");
 		generateStatsMsgs();
 	}
