@@ -6,7 +6,7 @@
  *
  * File begun on 2007-07-20 by RGerhards (extracted from syslogd.c)
  *
- * Copyright 2007-2012 Adiscon GmbH.
+ * Copyright 2007-2013 Adiscon GmbH.
  *
  * This file is part of rsyslog.
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -55,18 +55,22 @@ DEF_OMOD_STATIC_DATA
 DEFobjCurrIf(errmsg)
 
 typedef struct _instanceData {
-	MYSQL	*f_hmysql;			/* handle to MySQL */
 	char	dbsrv[MAXHOSTNAMELEN+1];	/* IP or hostname of DB server*/ 
 	unsigned int dbsrvPort;		/* port of MySQL server */
 	char	dbname[_DB_MAXDBLEN+1];	/* DB name */
 	char	dbuid[_DB_MAXUNAMELEN+1];	/* DB user */
 	char	dbpwd[_DB_MAXPWDLEN+1];	/* DB user's password */
-	unsigned uLastMySQLErrno;		/* last errno returned by MySQL or 0 if all is well */
-	uchar * f_configfile;			/* MySQL Client Configuration File */
-	uchar * f_configsection;		/* MySQL Client Configuration Section */
-	uchar	*tplName;       /* format template to use */
+	uchar   *configfile;			/* MySQL Client Configuration File */
+	uchar   *configsection;		/* MySQL Client Configuration Section */
+	uchar	*tplName;			/* format template to use */
 } instanceData;
 
+typedef struct wrkrInstanceData {
+	instanceData *pData;
+	MYSQL	*hmysql;			/* handle to MySQL */
+	unsigned uLastMySQLErrno;		/* last errno returned by MySQL or 0 if all is well */
+} wrkrInstanceData_t;
+
 typedef struct configSettings_s {
 	int iSrvPort;				/* database server port */
 	uchar *pszMySQLConfigFile;	/* MySQL Client Configuration File */
@@ -104,6 +108,12 @@ CODESTARTcreateInstance
 ENDcreateInstance
 
 
+BEGINcreateWrkrInstance
+CODESTARTcreateWrkrInstance
+	pWrkrData->hmysql = NULL;
+ENDcreateWrkrInstance
+
+
 BEGINisCompatibleWithFeature
 CODESTARTisCompatibleWithFeature
 	if(eFeat == sFEATURERepeatedMsgReduction)
@@ -115,25 +125,28 @@ ENDisCompatibleWithFeature
  * MySQL connection.
  * Initially added 2004-10-28
  */
-static void closeMySQL(instanceData *pData)
+static void closeMySQL(wrkrInstanceData_t *pWrkrData)
 {
-	ASSERT(pData != NULL);
-
-	if(pData->f_hmysql != NULL) {	/* just to be on the safe side... */
-		mysql_close(pData->f_hmysql);	
-		pData->f_hmysql = NULL;
+	if(pWrkrData->hmysql != NULL) {	/* just to be on the safe side... */
+		mysql_close(pWrkrData->hmysql);	
+		pWrkrData->hmysql = NULL;
 	}
 }
 
 BEGINfreeInstance
 CODESTARTfreeInstance
-	free(pData->f_configfile);
-	free(pData->f_configsection);
+	free(pData->configfile);
+	free(pData->configsection);
 	free(pData->tplName);
-	closeMySQL(pData);
 ENDfreeInstance
 
 
+BEGINfreeWrkrInstance
+CODESTARTfreeWrkrInstance
+	closeMySQL(pWrkrData);
+ENDfreeWrkrInstance
+
+
 BEGINdbgPrintInstInfo
 CODESTARTdbgPrintInstInfo
 	/* nothing special here */
@@ -144,25 +157,23 @@ ENDdbgPrintInstInfo
  * We check if we have a valid MySQL handle. If not, we simply
  * report an error, but can not be specific. RGerhards, 2007-01-30
  */
-static void reportDBError(instanceData *pData, int bSilent)
+static void reportDBError(wrkrInstanceData_t *pWrkrData, int bSilent)
 {
 	char errMsg[512];
 	unsigned uMySQLErrno;
 
-	ASSERT(pData != NULL);
-
 	/* output log message */
 	errno = 0;
-	if(pData->f_hmysql == NULL) {
+	if(pWrkrData->hmysql == NULL) {
 		errmsg.LogError(0, NO_ERRCODE, "unknown DB error occured - could not obtain MySQL handle");
 	} else { /* we can ask mysql for the error description... */
-		uMySQLErrno = mysql_errno(pData->f_hmysql);
+		uMySQLErrno = mysql_errno(pWrkrData->hmysql);
 		snprintf(errMsg, sizeof(errMsg)/sizeof(char), "db error (%d): %s\n", uMySQLErrno,
-			mysql_error(pData->f_hmysql));
-		if(bSilent || uMySQLErrno == pData->uLastMySQLErrno)
+			mysql_error(pWrkrData->hmysql));
+		if(bSilent || uMySQLErrno == pWrkrData->uLastMySQLErrno)
 			dbgprintf("mysql, DBError(silent): %s\n", errMsg);
 		else {
-			pData->uLastMySQLErrno = uMySQLErrno;
+			pWrkrData->uLastMySQLErrno = uMySQLErrno;
 			errmsg.LogError(0, NO_ERRCODE, "%s", errMsg);
 		}
 	}
@@ -175,25 +186,26 @@ static void reportDBError(instanceData *pData, int bSilent)
  * MySQL connection.
  * Initially added 2004-10-28 mmeckelein
  */
-static rsRetVal initMySQL(instanceData *pData, int bSilent)
+static rsRetVal initMySQL(wrkrInstanceData_t *pWrkrData, int bSilent)
 {
+	instanceData *pData;
 	DEFiRet;
 
-	ASSERT(pData != NULL);
-	ASSERT(pData->f_hmysql == NULL);
-	pData->f_hmysql = mysql_init(NULL);
-	if(pData->f_hmysql == NULL) {
+	ASSERT(pWrkrData->hmysql == NULL);
+	pData = pWrkrData->pData;
+	pWrkrData->hmysql = mysql_init(NULL);
+	if(pWrkrData->hmysql == NULL) {
 		errmsg.LogError(0, RS_RET_SUSPENDED, "can not initialize MySQL handle");
 		iRet = RS_RET_SUSPENDED;
 	} else { /* we could get the handle, now on with work... */
-		mysql_options(pData->f_hmysql,MYSQL_READ_DEFAULT_GROUP,((pData->f_configsection!=NULL)?(char*)pData->f_configsection:"client"));
-		if(pData->f_configfile!=NULL){
+		mysql_options(pWrkrData->hmysql,MYSQL_READ_DEFAULT_GROUP,((pData->configsection!=NULL)?(char*)pData->configsection:"client"));
+		if(pData->configfile!=NULL){
 			FILE * fp;
-			fp=fopen((char*)pData->f_configfile,"r");
+			fp=fopen((char*)pData->configfile,"r");
 			int err=errno;
 			if(fp==NULL){
 				char msg[512];
-				snprintf(msg,sizeof(msg)/sizeof(char),"Could not open '%s' for reading",pData->f_configfile);
+				snprintf(msg,sizeof(msg)/sizeof(char),"Could not open '%s' for reading",pData->configfile);
 				if(bSilent) {
 					char errStr[512];
 					rs_strerror_r(err, errStr, sizeof(errStr));
@@ -202,17 +214,17 @@ static rsRetVal initMySQL(instanceData *pData, int bSilent)
 					errmsg.LogError(err,NO_ERRCODE,"mysql configuration error: %s\n",msg);
 			} else {
 				fclose(fp);
-				mysql_options(pData->f_hmysql,MYSQL_READ_DEFAULT_FILE,pData->f_configfile);
+				mysql_options(pWrkrData->hmysql,MYSQL_READ_DEFAULT_FILE,pData->configfile);
 			}
 		}
 		/* Connect to database */
-		if(mysql_real_connect(pData->f_hmysql, pData->dbsrv, pData->dbuid,
+		if(mysql_real_connect(pWrkrData->hmysql, pData->dbsrv, pData->dbuid,
 				      pData->dbpwd, pData->dbname, pData->dbsrvPort, NULL, 0) == NULL) {
-			reportDBError(pData, bSilent);
-			closeMySQL(pData); /* ignore any error we may get */
+			reportDBError(pWrkrData, bSilent);
+			closeMySQL(pWrkrData); /* ignore any error we may get */
 			ABORT_FINALIZE(RS_RET_SUSPENDED);
 		}
-		mysql_autocommit(pData->f_hmysql, 0);
+		mysql_autocommit(pWrkrData->hmysql, 0);
 	}
 
 finalize_it:
@@ -224,35 +236,32 @@ finalize_it:
  * to an established MySQL session.
  * Initially added 2004-10-28 mmeckelein
  */
-rsRetVal writeMySQL(uchar *psz, instanceData *pData)
+rsRetVal writeMySQL(wrkrInstanceData_t *pWrkrData, uchar *psz)
 {
 	DEFiRet;
 
-	ASSERT(psz != NULL);
-	ASSERT(pData != NULL);
-
 	/* see if we are ready to proceed */
-	if(pData->f_hmysql == NULL) {
-		CHKiRet(initMySQL(pData, 0));
+	if(pWrkrData->hmysql == NULL) {
+		CHKiRet(initMySQL(pWrkrData, 0));
 		
 	}
 
 	/* try insert */
-	if(mysql_query(pData->f_hmysql, (char*)psz)) {
+	if(mysql_query(pWrkrData->hmysql, (char*)psz)) {
 		/* error occured, try to re-init connection and retry */
-		closeMySQL(pData); /* close the current handle */
-		CHKiRet(initMySQL(pData, 0)); /* try to re-open */
-		if(mysql_query(pData->f_hmysql, (char*)psz)) { /* re-try insert */
+		closeMySQL(pWrkrData); /* close the current handle */
+		CHKiRet(initMySQL(pWrkrData, 0)); /* try to re-open */
+		if(mysql_query(pWrkrData->hmysql, (char*)psz)) { /* re-try insert */
 			/* we failed, giving up for now */
-			reportDBError(pData, 0);
-			closeMySQL(pData); /* free ressources */
+			reportDBError(pWrkrData, 0);
+			closeMySQL(pWrkrData); /* free ressources */
 			ABORT_FINALIZE(RS_RET_SUSPENDED);
 		}
 	}
 
 finalize_it:
 	if(iRet == RS_RET_OK) {
-		pData->uLastMySQLErrno = 0; /* reset error for error supression */
+		pWrkrData->uLastMySQLErrno = 0; /* reset error for error supression */
 	}
 
 	RETiRet;
@@ -261,28 +270,28 @@ finalize_it:
 
 BEGINtryResume
 CODESTARTtryResume
-	if(pData->f_hmysql == NULL) {
-		iRet = initMySQL(pData, 1);
+	if(pWrkrData->hmysql == NULL) {
+		iRet = initMySQL(pWrkrData, 1);
 	}
 ENDtryResume
 
 BEGINbeginTransaction
 CODESTARTbeginTransaction
-	CHKiRet(writeMySQL((uchar*)"START TRANSACTION", pData));
+	CHKiRet(writeMySQL(pWrkrData, (uchar*)"START TRANSACTION"));
 finalize_it:
 ENDbeginTransaction
 
 BEGINdoAction
 CODESTARTdoAction
 	dbgprintf("\n");
-	CHKiRet(writeMySQL(ppString[0], pData));
+	CHKiRet(writeMySQL(pWrkrData, ppString[0]));
 	iRet = RS_RET_DEFER_COMMIT;
 finalize_it:
 ENDdoAction
 
 BEGINendTransaction
 CODESTARTendTransaction
-	if (mysql_commit(pData->f_hmysql) != 0)	{	
+	if(mysql_commit(pWrkrData->hmysql) != 0)	{	
 		dbgprintf("mysql server error: transaction not committed\n");		
 		iRet = RS_RET_SUSPENDED;
 	}
@@ -293,10 +302,9 @@ static inline void
 setInstParamDefaults(instanceData *pData)
 {
 	pData->dbsrvPort = 0;
-	pData->f_configfile = NULL;
-	pData->f_configsection = NULL;
+	pData->configfile = NULL;
+	pData->configsection = NULL;
 	pData->tplName = NULL;
-	pData->f_hmysql = NULL; /* initialize, but connect only on first message (important for queued mode!) */
 }
 
 
@@ -338,9 +346,9 @@ CODESTARTnewActInst
 			strncpy(pData->dbpwd, cstr, sizeof(pData->dbpwd));
 			free(cstr);
 		} else if(!strcmp(actpblk.descr[i].name, "mysqlconfig.file")) {
-			pData->f_configfile = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+			pData->configfile = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "mysqlconfig.section")) {
-			pData->f_configsection = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+			pData->configsection = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "template")) {
 			pData->tplName = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else {
@@ -424,9 +432,8 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 		ABORT_FINALIZE(RS_RET_INVALID_PARAMS);
 	} else {
 		pData->dbsrvPort = (unsigned) cs.iSrvPort;	/* set configured port */
-		pData->f_configfile = cs.pszMySQLConfigFile;
-		pData->f_configsection = cs.pszMySQLConfigSection;
-		pData->f_hmysql = NULL; /* initialize, but connect only on first message (important for queued mode!) */
+		pData->configfile = cs.pszMySQLConfigFile;
+		pData->configsection = cs.pszMySQLConfigSection;
 	}
 
 CODE_STD_FINALIZERparseSelectorAct
@@ -446,6 +453,7 @@ ENDmodExit
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
+CODEqueryEtryPt_STD_OMOD8_QUERIES
 CODEqueryEtryPt_STD_CONF2_OMOD_QUERIES
 CODEqueryEtryPt_TXIF_OMOD_QUERIES /* we support the transactional interface! */
 ENDqueryEtryPt
@@ -484,7 +492,7 @@ CODEmodInit_QueryRegCFSLineHdlr
 	   mysql_server_init(0, NULL, NULL)
 #	endif
 	                                   ) {
-		errmsg.LogError(0, NO_ERRCODE, "ommysql: mysql_server_init() failed, plugin "
+		errmsg.LogError(0, NO_ERRCODE, "ommysql: intializing mysql client failed, plugin "
 		                "can not run");
 		ABORT_FINALIZE(RS_RET_ERR);
 	}
