@@ -22,7 +22,7 @@
  * NOTE: read comments in module-template.h to understand how this file
  *       works!
  *
- * Copyright 2007-2012 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2013 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -63,7 +63,6 @@ MODULE_CNFNAME("omtesting")
  */
 DEF_OMOD_STATIC_DATA
 
-
 typedef struct _instanceData {
 	enum { MD_SLEEP, MD_FAIL, MD_RANDFAIL, MD_ALWAYS_SUSPEND }
 		mode;
@@ -76,6 +75,10 @@ typedef struct _instanceData {
 	int	iCurrRetries;
 } instanceData;
 
+typedef struct wrkrInstanceData {
+	instanceData *pData;
+} wrkrInstanceData_t;
+
 typedef struct configSettings_s {
 	int bEchoStdout;	/* echo non-failed messages to stdout */
 } configSettings_t;
@@ -93,6 +96,11 @@ CODESTARTcreateInstance
 ENDcreateInstance
 
 
+BEGINcreateWrkrInstance
+CODESTARTcreateWrkrInstance
+ENDcreateWrkrInstance
+
+
 BEGINdbgPrintInstInfo
 CODESTARTdbgPrintInstInfo
 	dbgprintf("Action delays rule by %d second(s) and %d millisecond(s)\n",
@@ -170,11 +178,11 @@ static rsRetVal doRandFail(void)
 BEGINtryResume
 CODESTARTtryResume
 	dbgprintf("omtesting tryResume() called\n");
-	switch(pData->mode) {
+	switch(pWrkrData->pData->mode) {
 		case MD_SLEEP:
 			break;
 		case MD_FAIL:
-			iRet = doFailOnResume(pData);
+			iRet = doFailOnResume(pWrkrData->pData);
 			break;
 		case MD_RANDFAIL:
 			iRet = doRandFail();
@@ -187,8 +195,10 @@ ENDtryResume
 
 
 BEGINdoAction
+	instanceData *pData;
 CODESTARTdoAction
 	dbgprintf("omtesting received msg '%s'\n", ppString[0]);
+	pData = pWrkrData->pData;
 	switch(pData->mode) {
 		case MD_SLEEP:
 			iRet = doSleep(pData);
@@ -220,6 +230,11 @@ CODESTARTfreeInstance
 ENDfreeInstance
 
 
+BEGINfreeWrkrInstance
+CODESTARTfreeWrkrInstance
+ENDfreeWrkrInstance
+
+
 BEGINparseSelectorAct
 	int i;
 	uchar szBuf[1024];
@@ -313,6 +328,7 @@ ENDmodExit
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
+CODEqueryEtryPt_STD_OMOD8_QUERIES
 CODEqueryEtryPt_STD_CONF2_CNFNAME_QUERIES 
 ENDqueryEtryPt
 
