@@ -5,25 +5,23 @@
  * in a useful manner. It is still undecided if all functions will continue
  * to stay here or some will be moved into parser modules (once we have them).
  *
- * Copyright 2008 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2012 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
- * The rsyslog runtime library is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * The rsyslog runtime library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with the rsyslog runtime library.  If not, see <http://www.gnu.org/licenses/>.
- *
- * A copy of the GPL can be found in the file "COPYING" in this distribution.
- * A copy of the LGPL can be found in the file "COPYING.LESSER" in this distribution.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ * 
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *       -or-
+ *       see COPYING.ASL20 in the source distribution
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "config.h"
@@ -63,8 +61,10 @@ timeval2syslogTime(struct timeval *tp, struct syslogTime *t)
 	struct tm *tm;
 	struct tm tmBuf;
 	long lBias;
+	time_t secs;
 
-	tm = localtime_r((time_t*) &(tp->tv_sec), &tmBuf);
+	secs = tp->tv_sec;
+	tm = localtime_r(&secs, &tmBuf);
 
 	t->year = tm->tm_year + 1900;
 	t->month = tm->tm_mon + 1;
@@ -79,7 +79,7 @@ timeval2syslogTime(struct timeval *tp, struct syslogTime *t)
 		/* Solaris uses a different method of exporting the time zone.
 		 * It is UTC - localtime, which is the opposite sign of mins east of GMT.
 		 */
-		lBias = -(daylight ? altzone : timezone);
+		lBias = -(tm->tm_isdst ? altzone : timezone);
 #	elif defined(__hpux)
 		lBias = tz.tz_dsttime ? - tz.tz_minuteswest : 0;
 #	else
@@ -182,12 +182,13 @@ getTime(time_t *ttSeconds)
  * 		  the method always returns zero.
  * \retval The number parsed.
  */
-static int srSLMGParseInt32(uchar** ppsz, int *pLenStr)
+static inline int
+srSLMGParseInt32(uchar** ppsz, int *pLenStr)
 {
 	register int i;
 
 	i = 0;
-	while(*pLenStr > 0 && isdigit((int) **ppsz)) {
+	while(*pLenStr > 0 && **ppsz >= '0' && **ppsz <= '9') {
 		i = i * 10 + **ppsz - '0';
 		++(*ppsz);
 		--(*pLenStr);
@@ -305,8 +306,10 @@ ParseTIMESTAMP3339(struct syslogTime *pTime, uchar** ppszTS, int *pLenStr)
 		if(OffsetHour < 0 || OffsetHour > 23)
 			ABORT_FINALIZE(RS_RET_INVLD_TIME);
 
-		if(lenStr == 0 || *pszTS++ != ':')
+		if(lenStr == 0 || *pszTS != ':')
 			ABORT_FINALIZE(RS_RET_INVLD_TIME);
+		--lenStr;
+		pszTS++;
 		OffsetMinute = srSLMGParseInt32(&pszTS, &lenStr);
 		if(OffsetMinute < 0 || OffsetMinute > 59)
 			ABORT_FINALIZE(RS_RET_INVLD_TIME);
@@ -623,6 +626,15 @@ finalize_it:
 	RETiRet;
 }
 
+void
+applyDfltTZ(struct syslogTime *pTime, char *tz)
+{
+	pTime->OffsetMode = tz[0];
+	pTime->OffsetHour = (tz[1] - '0') * 10 + (tz[2] - '0');
+	pTime->OffsetMinute = (tz[4] - '0') * 10 + (tz[5] - '0');
+
+}
+
 /*******************************************************************
  * END CODE-LIBLOGGING                                             *
  *******************************************************************/
@@ -852,6 +864,131 @@ int formatTimestamp3164(struct syslogTime *ts, char* pBuf, int bBuggyDay)
 }
 
 
+/**
+ * convert syslog timestamp to time_t
+ */
+time_t syslogTime2time_t(struct syslogTime *ts)
+{
+	long MonthInDays, NumberOfYears, NumberOfDays, i;
+	int utcOffset;
+	time_t TimeInUnixFormat;
+
+	/* Counting how many Days have passed since the 01.01 of the
+	 * selected Year (Month level), according to the selected Month*/
+
+	switch(ts->month)
+	{
+		case 1:
+			MonthInDays = 0;         //until 01 of January
+			break;
+		case 2:
+			MonthInDays = 31;        //until 01 of February - leap year handling down below!
+			break;
+		case 3:
+			MonthInDays = 59;        //until 01 of March
+			break;
+		case 4:
+			MonthInDays = 90;        //until 01 of April
+			break;
+		case 5:
+			MonthInDays = 120;       //until 01 of Mai
+			break;
+		case 6:
+			MonthInDays = 151;       //until 01 of June
+			break;
+		case 7:
+			MonthInDays = 181;       //until 01 of July
+			break;
+		case 8:
+			MonthInDays = 212;       //until 01 of August
+			break;
+		case 9:
+			MonthInDays = 243;       //until 01 of September
+			break;
+		case 10:
+			MonthInDays = 273;       //until 01 of Oktober
+			break;
+		case 11:
+			MonthInDays = 304;       //until 01 of November
+			break;
+		case 12:
+			MonthInDays = 334;       //until 01 of December
+			break;
+		default: /* this cannot happen (and would be a program error)
+		          * but we need the code to keep the compiler silent.
+			  */
+			MonthInDays = 0;	/* any value fits ;) */
+			break;
+	}	
+
+
+	/*	1) Counting how many Years have passed since 1970
+		2) Counting how many Days have passed since the 01.01 of the selected Year
+			(Day level) according to the Selected Month and Day. Last day doesn't count,
+			it should be until last day
+		3) Calculating this period (NumberOfDays) in seconds*/
+
+	NumberOfYears = ts->year - 1970;										
+	NumberOfDays = MonthInDays + ts->day - 1;
+	TimeInUnixFormat = NumberOfYears * 31536000 + NumberOfDays * 86400;
+
+	/* Now we need to adjust the number of years for leap
+	 * year processing. If we are in Jan or Feb, this year
+	 * will never be considered - because we haven't arrived
+	 * at then end of Feb right now. [Feb, 29th in a leap year
+	 * is handled correctly, because the day (29) is correctly
+	 * added to the date serial]
+	 */
+	if(ts->month < 3)
+		NumberOfYears--;
+
+	/*...AND ADDING ONE DAY FOR EACH YEAR WITH 366 DAYS
+	 * note that we do not handle 2000 any special, as it was a
+	 * leap year. The current code works OK until 2100, when it will
+	 * break. As we do not process future dates, we accept that fate...
+	 * the whole thing could be refactored by a table-based approach.
+	 */
+	for(i = 1;i <= NumberOfYears; i++)
+	{
+	/*	If i = 2 we have 1972, which was a Year with 366 Days
+		and if (i + 2) Mod (4) = 0 we have a Year after 1972
+		which is also a Year with 366 Days (repeated every 4 Years) */
+		if ((i == 2) || (((i + 2) % 4) == 0))
+		{	/*Year with 366 Days!!!*/
+			TimeInUnixFormat += 86400;
+		}	
+	}
+
+	/*Add Hours, minutes and seconds */
+	TimeInUnixFormat += ts->hour*60*60;
+	TimeInUnixFormat += ts->minute*60;
+	TimeInUnixFormat += ts->second;
+	/* do UTC offset */
+	utcOffset = ts->OffsetHour*3600 + ts->OffsetMinute*60;
+	if(ts->OffsetMode == '+')
+		utcOffset *= -1; /* if timestamp is ahead, we need to "go back" to UTC */
+	TimeInUnixFormat += utcOffset;
+	return TimeInUnixFormat;
+}
+
+
+/**
+ * format a timestamp as a UNIX timestamp; subsecond resolution is
+ * discarded.
+ * Note that this code can use some refactoring. I decided to use it
+ * because mktime() requires an upfront TZ update as it works on local
+ * time. In any case, it is worth reconsidering to move to mktime() or
+ * some other method.
+ * Important: pBuf must point to a buffer of at least 11 bytes.
+ * rgerhards, 2012-03-29
+ */
+int formatTimestampUnix(struct syslogTime *ts, char *pBuf)
+{
+	snprintf(pBuf, 11, "%u", (unsigned) syslogTime2time_t(ts));
+	return 11;
+}
+
+
 /* queryInterface function
  * rgerhards, 2008-03-05
  */
@@ -876,6 +1013,8 @@ CODESTARTobjQueryInterface(datetime)
 	pIf->formatTimestampSecFrac = formatTimestampSecFrac;
 	pIf->formatTimestamp3339 = formatTimestamp3339;
 	pIf->formatTimestamp3164 = formatTimestamp3164;
+	pIf->formatTimestampUnix = formatTimestampUnix;
+	pIf->syslogTime2time_t = syslogTime2time_t;
 finalize_it:
 ENDobjQueryInterface(datetime)
 
