@@ -7,7 +7,7 @@
  *
  * Module begun 2008-04-16 by Rainer Gerhards
  *
- * Copyright 2008-2014 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2013 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
@@ -31,7 +31,6 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include <pthread.h>
 #include <assert.h>
@@ -44,7 +43,6 @@
 #include "prop.h"
 #include "atomic.h"
 #include "errmsg.h"
-#include "action.h"
 #include "rainerscript.h"
 #include "net.h"
 
@@ -63,13 +61,7 @@ DEFobjCurrIf(net)
  * For this object, these variables are obviously what makes the "meat" of the
  * class...
  */
-int glblDebugOnShutdown = 0;	/* start debug log when we are shut down */
-
 static struct cnfobj *mainqCnfObj = NULL;/* main queue object, to be used later in startup sequence */
-int bProcessInternalMessages = 1;	/* Should rsyslog itself process internal messages?
-					 * 1 - yes
-					 * 0 - send them to libstdlog (e.g. to push to journal)
-					 */
 static uchar *pszWorkDir = NULL;
 static int bOptimizeUniProc = 1;	/* enable uniprocessor optimizations */
 static int bParseHOSTNAMEandTAG = 1;	/* parser modification (based on startup params!) */
@@ -109,15 +101,10 @@ static struct cnfparamdescr cnfparamdescr[] = {
 	{ "dropmsgswithmaliciousdnsptrrecords", eCmdHdlrBinary, 0 },
 	{ "localhostname", eCmdHdlrGetWord, 0 },
 	{ "preservefqdn", eCmdHdlrBinary, 0 },
-	{ "debug.onshutdown", eCmdHdlrBinary, 0 },
-	{ "debug.logfile", eCmdHdlrString, 0 },
 	{ "defaultnetstreamdrivercafile", eCmdHdlrString, 0 },
 	{ "defaultnetstreamdriverkeyfile", eCmdHdlrString, 0 },
 	{ "defaultnetstreamdriver", eCmdHdlrString, 0 },
 	{ "maxmessagesize", eCmdHdlrSize, 0 },
-	{ "action.reportsuspension", eCmdHdlrBinary, 0 },
-	{ "action.reportsuspensioncontinuation", eCmdHdlrBinary, 0 },
-	{ "processinternalmessages", eCmdHdlrBinary, 0 }
 };
 static struct cnfparamblk paramblk =
 	{ CNFPARAMBLK_VERSION,
@@ -489,14 +476,6 @@ GetWorkDir(void)
 	return(pszWorkDir == NULL ? (uchar*) "" : pszWorkDir);
 }
 
-/* return the "raw" working directory, which means
- * NULL if unset.
- */
-const uchar *
-glblGetWorkDirRaw(void)
-{
-	return pszWorkDir;
-}
 
 /* return the current default netstream driver */
 static uchar*
@@ -645,23 +624,9 @@ glblPrepCnf(void)
 void
 glblProcessCnf(struct cnfobj *o)
 {
-	int i;
-
 	cnfparamvals = nvlstGetParams(o->nvlst, &paramblk, cnfparamvals);
 	dbgprintf("glbl param blk after glblProcessCnf:\n");
 	cnfparamsPrint(&paramblk, cnfparamvals);
-
-	/* The next thing is a bit hackish and should be changed in higher
-	 * versions. There are a select few parameters which we need to
-	 * act on immediately. These are processed here.
-	 */
-	for(i = 0 ; i < paramblk.nParams ; ++i) {
-		if(!cnfparamvals[i].bUsed)
-			continue;
-		if(!strcmp(paramblk.descr[i].name, "processinternalmessages")) {
-			bProcessInternalMessages = (int) cnfparamvals[i].val.d.n;
-		}
-	}
 }
 
 /* Set mainq parameters. Note that when this is not called, we'll use the
@@ -692,10 +657,6 @@ glblDestructMainqCnfObj()
 	mainqCnfObj = NULL;
 }
 
-
-/* This processes the "regular" parameters which are to be set after the
- * config has been fully loaded.
- */
 void
 glblDoneLoadCnf(void)
 {
@@ -732,34 +693,14 @@ glblDoneLoadCnf(void)
 		} else if(!strcmp(paramblk.descr[i].name,
 				"dropmsgswithmaliciousdnsptrrecords")) {
 			bDropMalPTRMsgs = (int) cnfparamvals[i].val.d.n;
-		} else if(!strcmp(paramblk.descr[i].name, "action.reportsuspension")) {
-			bActionReportSuspension = (int) cnfparamvals[i].val.d.n;
-		} else if(!strcmp(paramblk.descr[i].name, "action.reportsuspensioncontinuation")) {
-			bActionReportSuspensionCont = (int) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "maxmessagesize")) {
 			iMaxLine = (int) cnfparamvals[i].val.d.n;
-		} else if(!strcmp(paramblk.descr[i].name, "debug.onshutdown")) {
-			glblDebugOnShutdown = (int) cnfparamvals[i].val.d.n;
-			errmsg.LogError(0, RS_RET_OK, "debug: onShutdown set to %d", glblDebugOnShutdown);
-		} else if(!strcmp(paramblk.descr[i].name, "debug.logfile")) {
-			if(pszAltDbgFileName == NULL) {
-				pszAltDbgFileName = es_str2cstr(cnfparamvals[i].val.d.estr, NULL);
-				if((altdbg = open(pszAltDbgFileName, O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_CLOEXEC, S_IRUSR|S_IWUSR)) == -1) {
-					errmsg.LogError(0, RS_RET_ERR, "debug log file '%s' could not be opened", pszAltDbgFileName);
-				}
-			}
-			errmsg.LogError(0, RS_RET_OK, "debug log file is '%s', fd %d", pszAltDbgFileName, altdbg);
 		} else {
 			dbgprintf("glblDoneLoadCnf: program error, non-handled "
 			  "param '%s'\n", paramblk.descr[i].name);
 		}
 	}
-
-	if(glblDebugOnShutdown && Debug != DEBUG_FULL) {
-		Debug = DEBUG_ONDEMAND;
-		stddbg = -1;
-	}
-finalize_it:	return;
+finalize_it:	;
 }
 
 
