@@ -175,6 +175,44 @@ static rsRetVal freeInstance(void* pModData)\
 	RETiRet;\
 }
 
+/* createWrkrInstance()
+ */
+#define BEGINcreateWrkrInstance \
+static rsRetVal createWrkrInstance(wrkrInstanceData_t **ppWrkrData, instanceData *pData)\
+	{\
+	DEFiRet; /* store error code here */\
+	wrkrInstanceData_t *pWrkrData; /* use this to point to data elements */
+
+#define CODESTARTcreateWrkrInstance \
+	if((pWrkrData = calloc(1, sizeof(wrkrInstanceData_t))) == NULL) {\
+		*ppWrkrData = NULL;\
+		ENDfunc \
+		return RS_RET_OUT_OF_MEMORY;\
+	} \
+	pWrkrData->pData = pData;
+
+#define ENDcreateWrkrInstance \
+	*ppWrkrData = pWrkrData;\
+	RETiRet;\
+}
+
+/* freeWrkrInstance */
+#define BEGINfreeWrkrInstance \
+static rsRetVal freeWrkrInstance(void* pd)\
+{\
+	DEFiRet;\
+	wrkrInstanceData_t *pWrkrData;
+
+#define CODESTARTfreeWrkrInstance \
+	pWrkrData = (wrkrInstanceData_t*) pd;
+
+#define ENDfreeWrkrInstance \
+	if(pWrkrData != NULL)\
+		free(pWrkrData); /* we need to free this in any case */\
+	RETiRet;\
+}
+
+
 /* isCompatibleWithFeature()
  */
 #define BEGINisCompatibleWithFeature \
@@ -194,7 +232,7 @@ static rsRetVal isCompatibleWithFeature(syslogFeature __attribute__((unused)) eF
  * introduced in v4.3.3 -- rgerhards, 2009-04-27
  */
 #define BEGINbeginTransaction \
-static rsRetVal beginTransaction(instanceData __attribute__((unused)) *pData)\
+static rsRetVal beginTransaction(wrkrInstanceData_t __attribute__((unused)) *pWrkrData)\
 {\
 	DEFiRet;
 
@@ -209,7 +247,7 @@ static rsRetVal beginTransaction(instanceData __attribute__((unused)) *pData)\
  * introduced in v4.3.3 -- rgerhards, 2009-04-27
  */
 #define BEGINendTransaction \
-static rsRetVal endTransaction(instanceData __attribute__((unused)) *pData)\
+static rsRetVal endTransaction(wrkrInstanceData_t __attribute__((unused)) *pWrkrData)\
 {\
 	DEFiRet;
 
@@ -223,7 +261,7 @@ static rsRetVal endTransaction(instanceData __attribute__((unused)) *pData)\
 /* doAction()
  */
 #define BEGINdoAction \
-static rsRetVal doAction(uchar __attribute__((unused)) **ppString, unsigned __attribute__((unused)) iMsgOpts, instanceData __attribute__((unused)) *pData)\
+static rsRetVal doAction(uchar __attribute__((unused)) **ppString, unsigned __attribute__((unused)) iMsgOpts, wrkrInstanceData_t __attribute__((unused)) *pWrkrData)\
 {\
 	DEFiRet;
 
@@ -382,12 +420,12 @@ static rsRetVal newInpInst(struct nvlst *lst)\
  * rgerhard, 2007-08-02
  */
 #define BEGINtryResume \
-static rsRetVal tryResume(instanceData __attribute__((unused)) *pData)\
+static rsRetVal tryResume(wrkrInstanceData_t __attribute__((unused)) *pWrkrData)\
 {\
 	DEFiRet;
 
 #define CODESTARTtryResume \
-	assert(pData != NULL);
+	assert(pWrkrData != NULL);
 
 #define ENDtryResume \
 	RETiRet;\
@@ -467,6 +505,13 @@ static rsRetVal queryEtryPt(uchar *name, rsRetVal (**pEtryPoint)())\
 		*pEtryPoint = tryResume;\
 	}
 
+/* standard queries for output module interface in rsyslog v8+ */
+#define CODEqueryEtryPt_STD_OMOD8_QUERIES \
+	else if(!strcmp((char*) name, "createWrkrInstance")) {\
+		*pEtryPoint = createWrkrInstance;\
+	} else if(!strcmp((char*) name, "freeWrkrInstance")) {\
+		*pEtryPoint = freeWrkrInstance;\
+	}
 
 /* the following definition is queryEtryPt block that must be added
  * if an output module supports the transactional interface.
