@@ -139,6 +139,8 @@ struct modInfo_s {
 			rsRetVal (*parseSelectorAct)(uchar**, void**,omodStringRequest_t**);
 			rsRetVal (*newActInst)(uchar *modName, struct nvlst *lst, void **, omodStringRequest_t **);
 			rsRetVal (*SetShutdownImmdtPtr)(void *pData, void *pPtr);
+			rsRetVal (*createWrkrInstance)(void*ppWrkrData, void*pData);
+			rsRetVal (*freeWrkrInstance)(void*pWrkrData);
 		} om;
 		struct { /* data for library modules */
 		    	char dummy;
