@@ -501,7 +501,6 @@ dropPrivileges(rsconf_t *cnf)
 static inline void
 tellCoreConfigLoadDone(void)
 {
-	DBGPRINTF("telling rsyslog core that config load for %p is done\n", loadConf);
 	glblDoneLoadCnf();
 }
 
@@ -680,8 +679,12 @@ activateMainQueue()
 	mainqCnfObj = glbl.GetmainqCnfObj();
 	DBGPRINTF("activateMainQueue: mainq cnf obj ptr is %p\n", mainqCnfObj);
 	/* create message queue */
-	CHKiRet_Hdlr(createMainQueue(&pMsgQueue, UCHAR_CONSTANT("main Q"),
-		    		(mainqCnfObj == NULL) ? NULL : mainqCnfObj->nvlst)) {
+	iRet = createMainQueue(&pMsgQueue, UCHAR_CONSTANT("main Q"),
+		    		(mainqCnfObj == NULL) ? NULL : mainqCnfObj->nvlst);
+	if(iRet == RS_RET_OK) {
+		iRet = startMainQueue(pMsgQueue);
+	}
+	if(iRet != RS_RET_OK) {
 		/* no queue is fatal, we need to give up in that case... */
 		fprintf(stderr, "fatal error %d: could not create message queue - rsyslogd can not run!\n", iRet);
 		FINALIZE;
@@ -744,6 +747,7 @@ activate(rsconf_t *cnf)
 	tellModulesActivateConfig();
 	startInputModules();
 	CHKiRet(activateActions());
+	CHKiRet(activateRulesetQueues());
 	CHKiRet(activateMainQueue());
 	/* finally let the inputs run... */
 	runInputModules();
@@ -1221,6 +1225,7 @@ ourConf = loadConf; // TODO: remove, once ourConf is gone!
 		ABORT_FINALIZE(RS_RET_NO_ACTIONS);
 	}
 	tellLexEndParsing();
+	DBGPRINTF("Number of actions in this configuration: %d\n", iActionNbr);
 	rulesetOptimizeAll(loadConf);
 
 	tellCoreConfigLoadDone();
