@@ -49,6 +49,7 @@
 #define CONF_PROGNAME_BUFSIZE		16
 #define CONF_HOSTNAME_BUFSIZE		32
 #define CONF_PROP_BUFSIZE		16	/* should be close to sizeof(ptr) or lighly above it */
+#define CONF_IPARAMS_BUFSIZE		16	/* initial size of iparams array in wti (is automatically extended) */
 #define	CONF_MIN_SIZE_FOR_COMPRESS	60 	/* config param: minimum message size to try compression. The smaller
 						 * the message, the less likely is any compression gain. We check for
 						 * gain before we submit the message. But to do so we still need to
@@ -382,9 +383,6 @@ enum rsRetVal_				/** return value. All methods return this if not specified oth
 	RS_RET_LAST_ERRREPORT = -2355,/**< module does not emit more error messages as limit is reached */
 	RS_RET_READ_ERR = -2356,/**< read error occured (file i/o) */
 	RS_RET_CONF_PARSE_WARNING = -2357,/**< warning parsing config file */
-	RS_RET_CONF_WRN_FULLDLY_BELOW_HIGHWTR = -2358,/**< warning queue full delay mark below high wtr mark */
-	RS_RET_RESUMED = -2359,/**< status: action was resumed (used for reporting) */
-	RS_RET_RELP_NO_TLS = -2360,/**< librel does not support TLS (but TLS requested) */
 
 	/* RainerScript error messages (range 1000.. 1999) */
 	RS_RET_SYSVAR_NOT_FOUND = 1001, /**< system variable could not be found (maybe misspelled) */
@@ -508,13 +506,13 @@ void dbgprintf(char *, ...) __attribute__((format(printf, 1, 2)));
  * add them. -- rgerhards, 2008-04-17
  */
 extern uchar *glblModPath; /* module load path */
-extern void (*glblErrLogger)(const int, const int, const uchar*);
+extern rsRetVal (*glblErrLogger)(int, uchar*);
 
 /* some runtime prototypes */
 rsRetVal rsrtInit(char **ppErrObj, obj_if_t *pObjIF);
 rsRetVal rsrtExit(void);
 int rsrtIsInit(void);
-void rsrtSetErrLogger(void (*errLogger)(const int, const int, const uchar*));
+rsRetVal rsrtSetErrLogger(rsRetVal (*errLogger)(int, uchar*));
 
 /* this define below is (later) intended to be used to implement empty
  * structs. TODO: check if compilers supports this and, if not, define
