@@ -26,10 +26,14 @@
 #ifndef INCLUDED_TYPEDEFS_H
 #define INCLUDED_TYPEDEFS_H
 #include <stdint.h>
-#if defined(__FreeBSD__) || !defined(HAVE_LSEEK64)
+#if defined(__FreeBSD__)
 #include <sys/types.h>
 #endif
 
+#ifndef HAVE_LSEEK64
+#include <unistd.h>
+#endif
+
 /* some universal fixed size integer defines ... */
 typedef long long int64;
 typedef long long unsigned uint64;
