@@ -744,7 +744,8 @@ RunSelect(tcpsrv_t *pThis, nsd_epworkset_t workset[], size_t sizeWorkset)
 	pthread_cleanup_push(RunCancelCleanup, (void*) &pSel);
 	while(1) {
 		CHKiRet(nssel.Construct(&pSel));
-		// TODO: set driver
+		if(pThis->pszDrvrName != NULL)
+			CHKiRet(nssel.SetDrvrName(pSel, pThis->pszDrvrName));
 		CHKiRet(nssel.ConstructFinalize(pSel));
 
 		/* Add the TCP listen sockets to the list of read descriptors. */
@@ -860,7 +861,8 @@ Run(tcpsrv_t *pThis)
 	 * to prevent us from leaking anything. -- rgerhards, 20080-04-24
 	 */
 	if((localRet = nspoll.Construct(&pPoll)) == RS_RET_OK) {
-		// TODO: set driver
+		if(pThis->pszDrvrName != NULL)
+			CHKiRet(nspoll.SetDrvrName(pPoll, pThis->pszDrvrName));
 		localRet = nspoll.ConstructFinalize(pPoll);
 	}
 	if(localRet != RS_RET_OK) {
@@ -921,6 +923,7 @@ BEGINobjConstruct(tcpsrv) /* be sure to specify the object type also in END macr
 	pThis->ratelimitInterval = 0;
 	pThis->ratelimitBurst = 10000;
 	pThis->bUseFlowControl = 1;
+	pThis->pszDrvrName = NULL;
 ENDobjConstruct(tcpsrv)
 
 
@@ -933,12 +936,13 @@ tcpsrvConstructFinalize(tcpsrv_t *pThis)
 
 	/* prepare network stream subsystem */
 	CHKiRet(netstrms.Construct(&pThis->pNS));
+	if(pThis->pszDrvrName != NULL)
+		CHKiRet(netstrms.SetDrvrName(pThis->pNS, pThis->pszDrvrName));
 	CHKiRet(netstrms.SetDrvrMode(pThis->pNS, pThis->iDrvrMode));
 	if(pThis->pszDrvrAuthMode != NULL)
 		CHKiRet(netstrms.SetDrvrAuthMode(pThis->pNS, pThis->pszDrvrAuthMode));
 	if(pThis->pPermPeers != NULL)
 		CHKiRet(netstrms.SetDrvrPermPeers(pThis->pNS, pThis->pPermPeers));
-	// TODO: set driver!
 	CHKiRet(netstrms.ConstructFinalize(pThis->pNS));
 
 	/* set up listeners */
@@ -950,6 +954,8 @@ finalize_it:
 	if(iRet != RS_RET_OK) {
 		if(pThis->pNS != NULL)
 			netstrms.Destruct(&pThis->pNS);
+		errmsg.LogError(0, iRet, "tcpsrv could not create listener (inputname: '%s')",
+				(pThis->pszInputName == NULL) ? (uchar*)"*UNSET*" : pThis->pszInputName);
 	}
 	RETiRet;
 }
@@ -965,6 +971,7 @@ CODESTARTobjDestruct(tcpsrv)
 
 	if(pThis->pNS != NULL)
 		netstrms.Destruct(&pThis->pNS);
+	free(pThis->pszDrvrName);
 	free(pThis->pszDrvrAuthMode);
 	free(pThis->ppLstn);
 	free(pThis->ppLstnPort);
@@ -1182,6 +1189,16 @@ SetDrvrMode(tcpsrv_t *pThis, int iMode)
 	RETiRet;
 }
 
+static rsRetVal
+SetDrvrName(tcpsrv_t *pThis, uchar *name)
+{
+	DEFiRet;
+	ISOBJ_TYPE_assert(pThis, tcpsrv);
+	free(pThis->pszDrvrName);
+	CHKmalloc(pThis->pszDrvrName = ustrdup(name));
+finalize_it:
+	RETiRet;
+}
 
 /* set the driver authentication mode -- rgerhards, 2008-05-19 */
 static rsRetVal
@@ -1285,6 +1302,7 @@ CODESTARTobjQueryInterface(tcpsrv)
 	pIf->SetLstnMax = SetLstnMax;
 	pIf->SetDrvrMode = SetDrvrMode;
 	pIf->SetDrvrAuthMode = SetDrvrAuthMode;
+	pIf->SetDrvrName = SetDrvrName;
 	pIf->SetDrvrPermPeers = SetDrvrPermPeers;
 	pIf->SetCBIsPermittedHost = SetCBIsPermittedHost;
 	pIf->SetCBOpenLstnSocks = SetCBOpenLstnSocks;
