@@ -133,7 +133,8 @@ typedef struct s_dynaFileCacheEntry dynaFileCacheEntry;
 
 
 typedef struct _instanceData {
-	uchar	*fname;	/* file or template name (display only) */
+	pthread_mutex_t mutWrite; /* guard against multiple instances writing to single file */
+	uchar	*f_fname;	/* file or template name (display only) */
 	uchar 	*tplName;	/* name of assigned template */
 	strm_t	*pStrm;		/* our output stream */
 	char	bDynamicName;	/* 0 - static name, 1 - dynamic name (with properties) */
@@ -181,6 +182,20 @@ typedef struct _instanceData {
 	STATSCOUNTER_DEF(ctrMax, mutCtrMax);
 } instanceData;
 
+/* to build a linked list for temporary storage of lines while we cannot commit */
+typedef struct linebuf {
+	uchar *filename; /* for dynafiles, make go away */
+	uchar *ln;
+	unsigned iMsgOpts;
+	struct linebuf *pNext;
+} linebuf_t;
+
+typedef struct wrkrInstanceData {
+	instanceData *pData;
+	linebuf_t *pRoot;
+	linebuf_t *pLast;
+} wrkrInstanceData_t;
+
 
 typedef struct configSettings_s {
 	int iDynaFileCacheSize; /* max cache for dynamic files */
@@ -236,13 +251,9 @@ static struct cnfparamdescr actpdescr[] = {
 	{ "flushontxend", eCmdHdlrBinary, 0 }, /* legacy: omfileflushontxend */
 	{ "iobuffersize", eCmdHdlrSize, 0 }, /* legacy: omfileiobuffersize */
 	{ "dirowner", eCmdHdlrUID, 0 }, /* legacy: dirowner */
-	{ "dirownernum", eCmdHdlrInt, 0 }, /* legacy: dirownernum */
 	{ "dirgroup", eCmdHdlrGID, 0 }, /* legacy: dirgroup */
-	{ "dirgroupnum", eCmdHdlrInt, 0 }, /* legacy: dirgroupnum */
 	{ "fileowner", eCmdHdlrUID, 0 }, /* legacy: fileowner */
-	{ "fileownernum", eCmdHdlrInt, 0 }, /* legacy: fileownernum */
 	{ "filegroup", eCmdHdlrGID, 0 }, /* legacy: filegroup */
-	{ "filegroupnum", eCmdHdlrInt, 0 }, /* legacy: filegroupnum */
 	{ "dircreatemode", eCmdHdlrFileCreateMode, 0 }, /* legacy: dircreatemode */
 	{ "filecreatemode", eCmdHdlrFileCreateMode, 0 }, /* legacy: filecreatemode */
 	{ "failonchownfailure", eCmdHdlrBinary, 0 }, /* legacy: failonchownfailure */
@@ -294,11 +305,11 @@ CODESTARTdbgPrintInstInfo
 	if(pData->bDynamicName) {
 		dbgprintf("[dynamic]\n");
 	} else { /* regular file */
-		dbgprintf("%s%s\n", pData->fname,
+		dbgprintf("%s%s\n", pData->f_fname,
 			  (pData->pStrm == NULL) ? " (closed)" : "");
 	}
 
-	dbgprintf("\ttemplate='%s'\n", pData->fname);
+	dbgprintf("\ttemplate='%s'\n", pData->f_fname);
 	dbgprintf("\tuse async writer=%d\n", pData->bUseAsyncWriter);
 	dbgprintf("\tflush on TX end=%d\n", pData->bFlushOnTXEnd);
 	dbgprintf("\tflush interval=%d\n", pData->iFlushInterval);
@@ -418,7 +429,7 @@ static rsRetVal cflineParseOutchannel(instanceData *pData, uchar* p, omodStringR
 	}
 
 	/* OK, we finally got a correct template. So let's use it... */
-	pData->fname = ustrdup(pOch->pszFileTemplate);
+	pData->f_fname = ustrdup(pOch->pszFileTemplate);
 	pData->iSizeLimit = pOch->uSizeLimit;
 	/* WARNING: It is dangerous "just" to pass the pointer. As we
 	 * never rebuild the output channel description, this is acceptable here.
@@ -790,7 +801,7 @@ finalize_it:
 
 /* rgerhards 2004-11-11: write to a file output.  */
 static rsRetVal
-writeFile(uchar **ppString, unsigned iMsgOpts, instanceData *pData)
+writeFile(instanceData *pData, linebuf_t *linebuf)
 {
 	DEFiRet;
 
@@ -800,17 +811,17 @@ writeFile(uchar **ppString, unsigned iMsgOpts, instanceData *pData)
 	 * check if it still is ok or a new file needs to be created
 	 */
 	if(pData->bDynamicName) {
-		CHKiRet(prepareDynFile(pData, ppString[1], iMsgOpts));
+		CHKiRet(prepareDynFile(pData, linebuf->filename, linebuf->iMsgOpts));
 	} else { /* "regular", non-dynafile */
 		if(pData->pStrm == NULL) {
-			CHKiRet(prepareFile(pData, pData->fname));
+			CHKiRet(prepareFile(pData, pData->f_fname));
 			if(pData->pStrm == NULL) {
-				errmsg.LogError(0, RS_RET_NO_FILE_ACCESS, "Could not open output file '%s'", pData->fname);
+				errmsg.LogError(0, RS_RET_NO_FILE_ACCESS, "Could no open output file '%s'", pData->f_fname);
 			}
 		}
 	}
 
-	CHKiRet(doWrite(pData, ppString[0], strlen(CHAR_CONVERT(ppString[0]))));
+	CHKiRet(doWrite(pData, linebuf->ln, ustrlen(linebuf->ln)));
 
 finalize_it:
 	RETiRet;
@@ -892,13 +903,19 @@ ENDfreeCnf
 BEGINcreateInstance
 CODESTARTcreateInstance
 	pData->pStrm = NULL;
+	pthread_mutex_init(&pData->mutWrite, NULL);
 ENDcreateInstance
 
 
+BEGINcreateWrkrInstance
+CODESTARTcreateWrkrInstance
+ENDcreateWrkrInstance
+
+
 BEGINfreeInstance
 CODESTARTfreeInstance
 	free(pData->tplName);
-	free(pData->fname);
+	free(pData->f_fname);
 	if(pData->bDynamicName) {
 		dynaFileFreeCache(pData);
 	} else if(pData->pStrm != NULL)
@@ -917,9 +934,15 @@ CODESTARTfreeInstance
 		free(pData->cryprovName);
 		free(pData->cryprovNameFull);
 	}
+	pthread_mutex_destroy(&pData->mutWrite);
 ENDfreeInstance
 
 
+BEGINfreeWrkrInstance
+CODESTARTfreeWrkrInstance
+ENDfreeWrkrInstance
+
+
 BEGINtryResume
 CODESTARTtryResume
 ENDtryResume
@@ -930,8 +953,68 @@ CODESTARTbeginTransaction
 ENDbeginTransaction
 
 
+static rsRetVal
+bufferLine(wrkrInstanceData_t *pWrkrData, uchar *filename, uchar *line)
+{
+	linebuf_t *lb;
+	DEFiRet;
+
+	CHKmalloc(lb = (linebuf_t*) malloc(sizeof(linebuf_t)));
+dbgprintf("DDDD: filename '%s'\n", filename);
+	CHKmalloc(lb->filename = ustrdup(filename));
+	CHKmalloc(lb->ln = ustrdup(line));
+	lb->pNext = NULL;
+	if(pWrkrData->pRoot == NULL) {
+		pWrkrData->pRoot = pWrkrData->pLast = lb;
+	} else {
+		pWrkrData->pLast->pNext = lb;
+		pWrkrData->pLast = lb;
+	}
+finalize_it:
+	RETiRet;
+}
+
+static void
+submitCachedLines(wrkrInstanceData_t *pWrkrData, instanceData *pData)
+{
+	linebuf_t *curr, *todel;
+
+	for(curr = pWrkrData->pRoot ; curr != NULL ; ) {
+		DBGPRINTF("omfile: file to log to: %s\n", curr->filename);
+		DBGPRINTF("omfile: start of data: '%.128s'\n", curr->ln);
+		STATSCOUNTER_INC(pData->ctrRequests, pData->mutCtrRequests);
+		writeFile(pData, curr);
+
+		todel = curr;
+		curr = curr->pNext;
+		free(todel->filename);
+		free(todel->ln);
+		free(todel);
+	}
+	pWrkrData->pRoot = NULL;
+}
+
+
+BEGINdoAction
+	instanceData *pData;
+CODESTARTdoAction
+	pData = pWrkrData->pData;
+dbgprintf("DDDD: bDynName %d, filename '%s'\n", pData->bDynamicName, ppString[1]);
+	iRet = bufferLine(pWrkrData, (pData->bDynamicName) ? ppString[1] : pData->f_fname,
+	                     ppString[0]);
+	if(iRet == RS_RET_OK)
+		iRet = RS_RET_DEFER_COMMIT;
+ENDdoAction
+
 BEGINendTransaction
+	instanceData *pData;
 CODESTARTendTransaction
+	pData = pWrkrData->pData;
+dbgprintf("omfile: waiting on write lock (pWrkrData %p)\n", pWrkrData);
+	pthread_mutex_lock(&pData->mutWrite);
+dbgprintf("omfile: aquired write lock    (pWrkrData %p)\n", pWrkrData);
+
+	submitCachedLines(pWrkrData, pData);
 	/* Note: pStrm may be NULL if there was an error opening the stream */
 	if(pData->bFlushOnTXEnd && pData->pStrm != NULL) {
 		/* if we have an async writer, it controls the flush via
@@ -942,29 +1025,16 @@ CODESTARTendTransaction
 			CHKiRet(strm.Flush(pData->pStrm));
 	}
 finalize_it:
+	pthread_mutex_unlock(&pData->mutWrite);
+dbgprintf("omfile: free write lock      (pWrkrData %p)\n", pWrkrData);
 ENDendTransaction
 
 
-BEGINdoAction
-CODESTARTdoAction
-	DBGPRINTF("file to log to: %s\n",
-		  (pData->bDynamicName) ? ppString[1] : pData->fname);
-	DBGPRINTF("omfile: start of data: '%.128s'\n", ppString[0]);
-	STATSCOUNTER_INC(pData->ctrRequests, pData->mutCtrRequests);
-	CHKiRet(writeFile(ppString, iMsgOpts, pData));
-	if(!bCoreSupportsBatching && pData->bFlushOnTXEnd) {
-		CHKiRet(strm.Flush(pData->pStrm));
-	}
-finalize_it:
-	if(iRet == RS_RET_OK)
-		iRet = RS_RET_DEFER_COMMIT;
-ENDdoAction
-
 
 static inline void
 setInstParamDefaults(instanceData *pData)
 {
-	pData->fname = NULL;
+	pData->f_fname = NULL;
 	pData->tplName = NULL;
 	pData->fileUID = -1;
 	pData->fileGID = -1;
@@ -1000,7 +1070,7 @@ setupInstStatsCtrs(instanceData *pData)
 	}
 
 	/* support statistics gathering */
-	snprintf((char*)ctrName, sizeof(ctrName), "dynafile cache %s", pData->fname);
+	snprintf((char*)ctrName, sizeof(ctrName), "dynafile cache %s", pData->f_fname);
 	ctrName[sizeof(ctrName)-1] = '\0'; /* be on the save side */
 	CHKiRet(statsobj.Construct(&(pData->stats)));
 	CHKiRet(statsobj.SetName(pData->stats, ctrName));
@@ -1152,20 +1222,12 @@ CODESTARTnewActInst
 			pData->iIOBufSize = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "dirowner")) {
 			pData->dirUID = (int) pvals[i].val.d.n;
-		} else if(!strcmp(actpblk.descr[i].name, "dirownernum")) {
-			pData->dirUID = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "dirgroup")) {
 			pData->dirGID = (int) pvals[i].val.d.n;
-		} else if(!strcmp(actpblk.descr[i].name, "dirgroupnum")) {
-			pData->dirGID = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "fileowner")) {
 			pData->fileUID = (int) pvals[i].val.d.n;
-		} else if(!strcmp(actpblk.descr[i].name, "fileownernum")) {
-			pData->fileUID = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "filegroup")) {
 			pData->fileGID = (int) pvals[i].val.d.n;
-		} else if(!strcmp(actpblk.descr[i].name, "filegroupnum")) {
-			pData->fileGID = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "dircreatemode")) {
 			pData->fDirCreateMode = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "filecreatemode")) {
@@ -1177,11 +1239,11 @@ CODESTARTnewActInst
 		} else if(!strcmp(actpblk.descr[i].name, "createdirs")) {
 			pData->bCreateDirs = (int) pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "file")) {
-			pData->fname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+			pData->f_fname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 			CODE_STD_STRING_REQUESTnewActInst(1)
 			pData->bDynamicName = 0;
 		} else if(!strcmp(actpblk.descr[i].name, "dynafile")) {
-			pData->fname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+			pData->f_fname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 			CODE_STD_STRING_REQUESTnewActInst(2)
 			pData->bDynamicName = 1;
 		} else if(!strcmp(actpblk.descr[i].name, "template")) {
@@ -1196,7 +1258,7 @@ CODESTARTnewActInst
 		}
 	}
 
-	if(pData->fname == NULL) {
+	if(pData->f_fname == NULL) {
 		errmsg.LogError(0, RS_RET_MISSING_CNFPARAMS, "omfile: either the \"file\" or "
 				"\"dynfile\" parameter must be given");
 		ABORT_FINALIZE(RS_RET_MISSING_CNFPARAMS);
@@ -1217,7 +1279,7 @@ CODESTARTnewActInst
 		/* "filename" is actually a template name, we need this as string 1. So let's add it
 		 * to the pOMSR. -- rgerhards, 2007-07-27
 		 */
-		CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->fname), OMSR_NO_RQD_TPL_OPTS));
+		CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->f_fname), OMSR_NO_RQD_TPL_OPTS));
 		// TODO: create unified code for this (legacy+v6 system)
 		/* we now allocate the cache table */
 		CHKmalloc(pData->dynCache = (dynaFileCacheEntry**)
@@ -1276,13 +1338,13 @@ CODESTARTparseSelectorAct
 		CODE_STD_STRING_REQUESTparseSelectorAct(2)
 		++p; /* eat '?' */
 		CHKiRet(cflineParseFileName(p, fname, *ppOMSR, 0, OMSR_NO_RQD_TPL_OPTS, getDfltTpl()));
-		pData->fname = ustrdup(fname);
+		pData->f_fname = ustrdup(fname);
 		pData->bDynamicName = 1;
 		pData->iCurrElt = -1;		  /* no current element */
 		/* "filename" is actually a template name, we need this as string 1. So let's add it
 		 * to the pOMSR. -- rgerhards, 2007-07-27
 		 */
-		CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->fname), OMSR_NO_RQD_TPL_OPTS));
+		CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->f_fname), OMSR_NO_RQD_TPL_OPTS));
 		/* we now allocate the cache table */
 		CHKmalloc(pData->dynCache = (dynaFileCacheEntry**)
 				calloc(cs.iDynaFileCacheSize, sizeof(dynaFileCacheEntry*)));
@@ -1292,7 +1354,7 @@ CODESTARTparseSelectorAct
 	case '.':
 		CODE_STD_STRING_REQUESTparseSelectorAct(1)
 		CHKiRet(cflineParseFileName(p, fname, *ppOMSR, 0, OMSR_NO_RQD_TPL_OPTS, getDfltTpl()));
-		pData->fname = ustrdup(fname);
+		pData->f_fname = ustrdup(fname);
 		pData->bDynamicName = 0;
 		break;
 	default:
@@ -1348,6 +1410,7 @@ static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __a
 
 BEGINdoHUP
 CODESTARTdoHUP
+	pthread_mutex_lock(&pData->mutWrite);
 	if(pData->bDynamicName) {
 		dynaFileFreeCacheEntries(pData);
 	} else {
@@ -1355,6 +1418,7 @@ CODESTARTdoHUP
 			closeFile(pData);
 		}
 	}
+	pthread_mutex_unlock(&pData->mutWrite);
 ENDdoHUP
 
 
@@ -1370,6 +1434,7 @@ ENDmodExit
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
+CODEqueryEtryPt_STD_OMOD8_QUERIES
 CODEqueryEtryPt_STD_CONF2_QUERIES
 CODEqueryEtryPt_STD_CONF2_setModCnf_QUERIES
 CODEqueryEtryPt_STD_CONF2_OMOD_QUERIES
@@ -1398,13 +1463,9 @@ INITLegCnfVars
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"omfileflushontxend", 0, eCmdHdlrBinary, NULL, &cs.bFlushOnTXEnd, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"omfileiobuffersize", 0, eCmdHdlrSize, NULL, &cs.iIOBufSize, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"dirowner", 0, eCmdHdlrUID, NULL, &cs.dirUID, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"dirownernum", 0, eCmdHdlrInt, NULL, &cs.dirUID, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"dirgroup", 0, eCmdHdlrGID, NULL, &cs.dirGID, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"dirgroupnum", 0, eCmdHdlrInt, NULL, &cs.dirGID, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"fileowner", 0, eCmdHdlrUID, NULL, &cs.fileUID, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"fileownernum", 0, eCmdHdlrInt, NULL, &cs.fileUID, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"filegroup", 0, eCmdHdlrGID, NULL, &cs.fileGID, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"filegroupnum", 0, eCmdHdlrInt, NULL, &cs.fileGID, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"dircreatemode", 0, eCmdHdlrFileCreateMode, NULL, &cs.fDirCreateMode, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"filecreatemode", 0, eCmdHdlrFileCreateMode, NULL, &cs.fCreateMode, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"createdirs", 0, eCmdHdlrBinary, NULL, &cs.bCreateDirs, STD_LOADABLE_MODULE_ID));
