@@ -21,7 +21,7 @@
  * For further information, please see http://www.rsyslog.com
  *
  * rsyslog - An Enhanced syslogd Replacement.
- * Copyright 2003-2014 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2003-2012 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -82,7 +82,6 @@
 #endif
 
 #include <signal.h>
-#include <liblogging/stdlog.h>
 
 #if HAVE_PATHS_H
 #include <paths.h>
@@ -418,10 +417,12 @@ finalize_it:
  * function is also passed to the runtime library as the generic error
  * message handler. -- rgerhards, 2008-04-17
  */
-void
-submitErrMsg(const int severity, const int iErr, const uchar *msg)
+rsRetVal
+submitErrMsg(int iErr, uchar *msg)
 {
-	logmsgInternal(iErr, LOG_SYSLOG|(severity & 0x07), msg, 0);
+	DEFiRet;
+	iRet = logmsgInternal(iErr, LOG_SYSLOG|LOG_ERR, msg, 0);
+	RETiRet;
 }
 
 
@@ -431,13 +432,11 @@ submitMsgWithDfltRatelimiter(msg_t *pMsg)
 	return ratelimitAddMsg(dflt_ratelimiter, NULL, pMsg);
 }
 
-/* This function logs a message to rsyslog itself, using its own
- * internal structures. This means external programs (like the
- * system journal) will never see this message.
+/* rgerhards 2004-11-09: the following is a function that can be used
+ * to log a message orginating from the syslogd itself.
  */
-static rsRetVal
-logmsgInternalSelf(const int iErr, const int pri, const size_t lenMsg,
-	const char *__restrict__ const msg, int flags)
+rsRetVal
+logmsgInternal(int iErr, int pri, uchar *msg, int flags)
 {
 	uchar pszTag[33];
 	msg_t *pMsg;
@@ -445,7 +444,7 @@ logmsgInternalSelf(const int iErr, const int pri, const size_t lenMsg,
 
 	CHKiRet(msgConstruct(&pMsg));
 	MsgSetInputName(pMsg, pInternalInputName);
-	MsgSetRawMsg(pMsg, (char*)msg, lenMsg);
+	MsgSetRawMsgWOSize(pMsg, (char*)msg);
 	MsgSetHOSTNAME(pMsg, glbl.GetLocalHostName(), ustrlen(glbl.GetLocalHostName()));
 	MsgSetRcvFrom(pMsg, glbl.GetLocalHostNameProp());
 	MsgSetRcvFromIP(pMsg, glbl.GetLocalHostIP());
@@ -465,6 +464,19 @@ logmsgInternalSelf(const int iErr, const int pri, const size_t lenMsg,
 	flags |= INTERNAL_MSG;
 	pMsg->msgFlags  = flags;
 
+	/* we now check if we should print internal messages out to stderr. This was
+	 * suggested by HKS as a way to help people troubleshoot rsyslog configuration
+	 * (by running it interactively. This makes an awful lot of sense, so I add
+	 * it here. -- rgerhards, 2008-07-28
+	 * Note that error messages can not be disable during a config verify. This
+	 * permits us to process unmodified config files which otherwise contain a
+	 * supressor statement.
+	 */
+	if(((Debug == DEBUG_FULL || !doFork) && ourConf->globals.bErrMsgToStderr) || iConfigVerify) {
+		if(LOG_PRI(pri) == LOG_ERR)
+			fprintf(stderr, "rsyslogd: %s\n", msg);
+	}
+
 	if(bHaveMainQueue == 0) { /* not yet in queued mode */
 		iminternalAddMsg(pMsg);
 	} else {
@@ -472,89 +484,32 @@ logmsgInternalSelf(const int iErr, const int pri, const size_t lenMsg,
 		 * message to the queue engine.
 		 */
 		ratelimitAddMsg(internalMsg_ratelimiter, NULL, pMsg);
+		//submitMsgWithDfltRatelimiter(pMsg);
 	}
 finalize_it:
 	RETiRet;
 }
 
 
-/* rgerhards 2004-11-09: the following is a function that can be used
- * to log a message orginating from the syslogd itself.
- */
-rsRetVal
-logmsgInternal(int iErr, int pri, const uchar *const msg, int flags)
-{
-	size_t lenMsg;
-	unsigned i;
-	char *bufModMsg = NULL; /* buffer for modified message, should we need to modify */
-	DEFiRet;
-
-	/* we first do a path the remove control characters that may have accidently
-	 * introduced (program error!). This costs performance, but we do not expect
-	 * to be called very frequently in any case ;) -- rgerhards, 2013-12-19.
-	 */
-	lenMsg = ustrlen(msg);
-	for(i = 0 ; i < lenMsg ; ++i) {
-		if(msg[i] < 0x20 || msg[i] == 0x7f) {
-			if(bufModMsg == NULL) {
-				CHKmalloc(bufModMsg = strdup((char*) msg));
-			}
-			bufModMsg[i] = ' ';
-		}
-	}
-
-	if(bProcessInternalMessages) {
-		CHKiRet(logmsgInternalSelf(iErr, pri, lenMsg,
-					   (bufModMsg == NULL) ? (char*)msg : bufModMsg,
-					   flags));
-	} else {
-		stdlog_log(NULL, LOG_PRI(pri), "%s",
-			   (bufModMsg == NULL) ? (char*)msg : bufModMsg);
-	}
-
-	/* we now check if we should print internal messages out to stderr. This was
-	 * suggested by HKS as a way to help people troubleshoot rsyslog configuration
-	 * (by running it interactively. This makes an awful lot of sense, so I add
-	 * it here. -- rgerhards, 2008-07-28
-	 * Note that error messages can not be disabled during a config verify. This
-	 * permits us to process unmodified config files which otherwise contain a
-	 * supressor statement.
-	 */
-	if(((Debug == DEBUG_FULL || !doFork) && ourConf->globals.bErrMsgToStderr) || iConfigVerify) {
-		if(LOG_PRI(pri) == LOG_ERR)
-			fprintf(stderr, "rsyslogd: %s\n", (bufModMsg == NULL) ? (char*)msg : bufModMsg);
-	}
-
-finalize_it:
-	free(bufModMsg);
-	RETiRet;
-}
-
-
 /* preprocess a batch of messages, that is ready them for actual processing. This is done
  * as a first stage and totally in parallel to any other worker active in the system. So
  * it helps us keep up the overall concurrency level.
  * rgerhards, 2010-06-09
  */
 static inline rsRetVal
-preprocessBatch(batch_t *pBatch) {
+preprocessBatch(batch_t *pBatch, int *pbShutdownImmediate) {
 	prop_t *ip;
 	prop_t *fqdn;
 	prop_t *localName;
 	prop_t *propFromHost = NULL;
 	prop_t *propFromHostIP = NULL;
-	int bSingleRuleset;
-	ruleset_t *batchRuleset; /* the ruleset used for all message inside the batch, if there is a single one */
 	int bIsPermitted;
 	msg_t *pMsg;
 	int i;
 	rsRetVal localRet;
 	DEFiRet;
 
-	bSingleRuleset = 1;
-	batchRuleset = (pBatch->nElem > 0) ? pBatch->pElem[0].pMsg->pRuleset : NULL;
-	
-	for(i = 0 ; i < pBatch->nElem  && !*(pBatch->pbShutdownImmediate) ; i++) {
+	for(i = 0 ; i < pBatch->nElem  && !*pbShutdownImmediate ; i++) {
 		pMsg = pBatch->pElem[i].pMsg;
 		if((pMsg->msgFlags & NEEDS_ACLCHK_U) != 0) {
 			DBGPRINTF("msgConsumer: UDP ACL must be checked for message (hostname-based)\n");
@@ -579,12 +534,8 @@ preprocessBatch(batch_t *pBatch) {
 				pBatch->eltState[i] = BATCH_STATE_DISC;
 			}
 		}
-		if(pMsg->pRuleset != batchRuleset)
-			bSingleRuleset = 0;
 	}
 
-	batchSetSingleRuleset(pBatch, bSingleRuleset);
-
 finalize_it:
 	if(propFromHost != NULL)
 		prop.Destruct(&propFromHost);
@@ -600,17 +551,16 @@ finalize_it:
  * for the main queue.
  */
 static rsRetVal
-msgConsumer(void __attribute__((unused)) *notNeeded, batch_t *pBatch, int *pbShutdownImmediate)
+msgConsumer(void __attribute__((unused)) *notNeeded, batch_t *pBatch, wti_t *pWti)
 {
 	DEFiRet;
 	assert(pBatch != NULL);
-	pBatch->pbShutdownImmediate = pbShutdownImmediate; /* TODO: move this to batch creation! */
-	preprocessBatch(pBatch);
-	ruleset.ProcessBatch(pBatch);
+	preprocessBatch(pBatch, pWti->pbShutdownImmediate);
+	ruleset.ProcessBatch(pBatch, pWti);
 //TODO: the BATCH_STATE_COMM must be set somewhere down the road, but we 
 //do not have this yet and so we emulate -- 2010-06-10
 int i;
-	for(i = 0 ; i < pBatch->nElem  && !*pbShutdownImmediate ; i++) {
+	for(i = 0 ; i < pBatch->nElem  && !*pWti->pbShutdownImmediate ; i++) {
 		pBatch->eltState[i] = BATCH_STATE_COMM;
 	}
 	RETiRet;
@@ -781,13 +731,6 @@ static void doDie(int sig)
 		abort();
 	}
 	bFinished = sig;
-	if(glblDebugOnShutdown) {
-		/* kind of hackish - set to 0, so that debug_swith will enable
-		 * and AND emit the "start debug log" message.
-		 */
-		debugging_on = 0;
-		debug_switch();
-	}
 #	undef MSG1
 #	undef MSG2
 }
@@ -1178,9 +1121,14 @@ rsRetVal createMainQueue(qqueue_t **ppQueue, uchar *pszQueueName, struct nvlst *
 		qqueueSetDefaultsRulesetQueue(*ppQueue);
 		qqueueApplyCnfParam(*ppQueue, lst);
 	}
+	RETiRet;
+}
 
-	/* ... and finally start the queue! */
-	CHKiRet_Hdlr(qqueueStart(*ppQueue)) {
+rsRetVal
+startMainQueue(qqueue_t *pQueue)
+{
+	DEFiRet;
+	CHKiRet_Hdlr(qqueueStart(pQueue)) {
 		/* no queue is fatal, we need to give up in that case... */
 		errmsg.LogError(0, iRet, "could not start (ruleset) main message queue"); \
 	}
@@ -1380,6 +1328,11 @@ static void printVersion(void)
 #else
 	printf("\t64bit Atomic operations supported:\tNo\n");
 #endif
+#ifdef	HAVE_JEMALLOC
+	printf("\tmemory allocator:\t\t\tjemalloc\n");
+#else
+	printf("\tmemory allocator:\t\t\tsystem default\n");
+#endif
 #ifdef	RTINST
 	printf("\tRuntime Instrumentation (slow code):\tYes\n");
 #else
@@ -1412,7 +1365,7 @@ InitGlobalClasses(void)
 	/* Intialize the runtime system */
 	pErrObj = "rsyslog runtime"; /* set in case the runtime errors before setting an object */
 	CHKiRet(rsrtInit(&pErrObj, &obj));
-	rsrtSetErrLogger(submitErrMsg);
+	CHKiRet(rsrtSetErrLogger(submitErrMsg)); /* set out error handler */
 
 	/* Now tell the system which classes we need ourselfs */
 	pErrObj = "glbl";
